/*
** main.c
**
** Ciotec Marian-Stefan 333CA
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include "lcd.h"

typedef struct {

	unsigned char x;
	unsigned char y;
	
} Coord;

/* main -- Program starts here */
int main(void)
{
	unsigned char i, j;
	unsigned char table[24][42];
	unsigned char part = 0xFF;
	unsigned char x_base, y_base;
	unsigned char button, dir, pressed_up = 0, pressed_down = 0, 
				  pressed_left = 0, pressed_right = 0, pressed_pause = 0;
	unsigned char menu_y = EASY;
	unsigned char hit = 0, dup = 0, level = EASY;
	unsigned int score = 0;
	unsigned int delay_time = 1000;
	unsigned char str[20];
	
	Coord snake[100];
	Coord bomb; 
	unsigned char size = 8;
	
	
	// Setup LCD
	lcd_init();
	lcd_contrast(0x40);
	lcd_clear();
	
	//Setup port C
	DDRC = 0x00;
	PORTC = 0xFF;
	
	MCUCR = _BV(JTD);
	MCUCR = _BV(JTD);
	
	// snake's initial position
	for (i = 0; i < size; i++) {
		snake[i].y = 1;
		snake[i].x = 21;
	}
	
	//Initialize table
	for (i = 0; i < 24; i++)
		for (j = 0; j < 42; j++)
			table[i][j] = EMPTY;
	
	button = NONE;
	
	//snake's initial direction
	dir = DOWN;
	
	//bomb initial position
	bomb.x = 10;
	bomb.y = 10;
	
	/* print logo */
	lcd_goto_xy(2, 2);
	lcd_str("Snake Game");
	while (1)
		if (bit_is_clear(PINC, PAUSE)) {
			lcd_clear();
			break;
		}
	_delay_ms(1000);
	
	while (1) {
		/* print menu */
		lcd_goto_xy(4, 1);
		lcd_str("Easy");
		lcd_goto_xy(4, 2);
		lcd_str("Medium");
		lcd_goto_xy(4,3);
		lcd_str("Hard");
		lcd_goto_xy(2, menu_y);
		lcd_chr(ARROW_RIGHT);
		
		if (bit_is_clear(PINC, UP)) {
			if (menu_y == EASY)
				menu_y = HARD;
			else
				menu_y--;
		}
		
		if (bit_is_clear(PINC, DOWN)) {
			if (menu_y == HARD)
				menu_y = EASY;
			else
				menu_y++;
		}
		
		if (bit_is_clear(PINC, RESET)) {
			lcd_clear();
			break;
		}
		
		_delay_ms(2000);
		lcd_clear();
	}
	_delay_ms(1000);
	
	level = menu_y;
	if (level == EASY)
		delay_time = 1000;
	else if (level == MEDIUM)
		delay_time = 800;
	else if (level == HARD)
		delay_time = 600;
	
	while (1) {
		
		if (bit_is_clear(PINC, UP)) {
			
			if (pressed_up == 0) {
				button = UP;
				pressed_up = 1;
			}
		}
		else {
		
			pressed_up = 0;
		}
		
		if (bit_is_clear(PINC, DOWN)) {
		
			if (pressed_down == 0) {
				button = DOWN;
				pressed_down = 1;
			}
		}
		else {
		
			pressed_down = 0;
		}
		
		if (bit_is_clear(PINC, LEFT)) {
					
			if (pressed_left == 0) {
				button = LEFT;
				pressed_left = 1;
			}
		}
		else {
		
			pressed_left = 0;
		}
		
		if (bit_is_clear(PINC, RIGHT)) {
		
			if (pressed_right == 0) {
				button = RIGHT;
				pressed_right = 1;
			}
		}
		else {
		
			pressed_right = 0;
		}
		
		if (bit_is_clear(PINC, PAUSE)) {
			if (pressed_pause == 0) {
				pressed_pause = 1;
				_delay_ms(2000);
				
				while (1)
					if (bit_is_clear(PINC, PAUSE))
						break;
			}
		}
		else {
			pressed_pause = 0;
		}
					
		switch (button) {
		
			case UP:
				if (dir != DOWN)
					dir = UP;
				break;
		
			case DOWN:
				if (dir != UP)
					dir = DOWN;
				break;
				
			case RIGHT:
				if (dir != LEFT)
					dir = RIGHT;
				break;
				
			case LEFT:
				if (dir != RIGHT)
					dir = LEFT;
				break;
		}
		
		/* Snake hit the bomb */
		if ((snake[size-1].x == bomb.x) && (snake[size-1].y == bomb.y)) {
			size++;
			hit = 1;
			snake[size-1].x = bomb.x;
			snake[size-1].y = bomb.y;
			score += 5;
		}
		else {
			hit = 0;
		}
		
		if (hit == 0) {
			/* Erase snake's tail */
			for (i = 0; i < size - 1; i++) {
				snake[i].x = snake[i+1].x;
				snake[i].y = snake[i+1].y;
			}
		}
		else {
			/* Generate new bomb */
			bomb.y = (unsigned char) rand() % 24;
			bomb.x = (unsigned char) rand() % 42;
			
			do {
				dup = 0;
				
				for (i = 0; i < size; i++)
					if (table[bomb.y][bomb.x] == BODY)
						dup = 1;
						
				if (dup == 1) {
					bomb.y = (unsigned char) rand() % 24;
					bomb.x = (unsigned char) rand() % 42;
				}
			} while (dup == 1);
		}
			
		
		switch(dir) {
			
			case UP:
			
				if (snake[size-1].y == 0)
					snake[size-1].y = 23;
				else
					snake[size-1].y--;
					
				break;
				
			case DOWN:
			
				if (snake[size-1].y == 23)
					snake[size-1].y = 0;
				else
					snake[size-1].y++;
					
				break;
				
			case RIGHT:
			
				if (snake[size-1].x == 41)
					snake[size-1].x = 0;
				else
					snake[size-1].x++;
					
				break;
				
			case LEFT:
			
				if (snake[size-1].x == 0)
					snake[size-1].x = 41;
				else
					snake[size-1].x--;
					
				break;
		}
		
		/* check if snake is dead */
		if (table[snake[size-1].y][snake[size-1].x] == BODY) {
			lcd_clear();
			lcd_goto_xy(3, 1);
			lcd_str("Game Over");
			sprintf(str, "Score %d", score * level);
			lcd_goto_xy(3, 3);
			lcd_str(str);
			break;
		}
		
		for (i = 0; i < 24; i++)
			for (j = 0; j < 42; j++)
				table[i][j] = EMPTY;
		
		for (i = 0; i < size; i++)
			table[snake[i].y][snake[i].x] = BODY;
			
		table[bomb.y][bomb.x] = BOMB;
		
		/* draw walls for medium and hard level */
		if (level == MEDIUM) {
			for (i = 0; i < 10; i++) {
				table[i][0] = BODY;
				table[i][41] = BODY;
			}
			for (i = 14; i < 24; i++) {
				table[i][0] = BODY;
				table[i][41] = BODY;
			}
			for (j = 0; j < 19; j++) {
				table[0][j] = BODY;
				table[23][j] = BODY;
			}
			for (j = 23; j < 42; j++) {
				table[0][j] = BODY;
				table[23][j] = BODY;
			}
		}
		
		if (level == HARD) {
			for (i = 0; i < 10; i++) {
				table[i][0] = BODY;
				table[i][41] = BODY;
			}
			for (i = 14; i < 24; i++) {
				table[i][0] = BODY;
				table[i][41] = BODY;
			}
			for (j = 0; j < 42; j++) {
				table[0][j] = BODY;
				table[23][j] = BODY;
				
				if (((j > 11) && (j < 19)) || ((j > 22) && (j < 30))) {
					table[8][j] = BODY;
					table[16][j] = BODY;
				}
			}
			for (i = 8; i <= 16; i++) {
				table[i][12] = BODY;
				table[i][30] = BODY;
			}
		}
		
		
		/* Draw table */
		for (i = 0; i < 24; i += 4) {
		
			y_base = (i * 2) / 8;
			
			for (j = 0; j < 42; j++) {
			
				part = 0x00;
				x_base = j * 2;
			
				if ((table[i][j] == BODY) || (table[i][j] == BOMB))
					part |= 0x03;
				
				if ((table[i+1][j] == BODY) || (table[i+1][j] == BOMB))
					part |= 0x0C;
					
				if ((table[i+2][j] == BODY) || (table[i+2][j] == BOMB))
					part |= 0x30;
					
				if ((table[i+3][j] == BODY) || (table[i+3][j] == BOMB))
					part |= 0xC0;
					
				lcd_draw_part_xy(part, x_base, y_base);
				lcd_draw_part_xy(part, x_base + 1, y_base);
					
					
			}
		}
		
		_delay_ms(delay_time);
		
	}
	
	return 0;
}
